import numpy as np
import os
import pickle


def unpickle(file):
    with open(file, 'rb') as fo:
        dict = pickle.load(fo, encoding='bytes')
    return dict


def read_cifar_batch(file):
    """
        La fonction lecture_cifar_batch : prenne en argument le chemin du répertoire contenant les données, et renvoyant:
    Une matrice Data de taille NxD où N correspond au nombre de données disponibles, et D à la dimension de ces données (nombre de valeurs numériques décrivant les données),
    Ainsi qu'un vecteur labels de taille N dont les valeurs correspondent au code de la classe de la donnée de même indice dans data. 
    Data et label sont objets numpy
    
    """

    dict = unpickle(file)
    data = dict[b'data'].astype(np.float32)
    labels = np.array(dict[b'labels'], dtype=np.int64)
    labels = labels.reshape(labels.shape[0])
    return data, labels

def read_cifar(path):
    """
    Cette fonction prend en argument le chemin du répertoire de toutes les batches y compris test_batch et renvoyant:
    - Matrice data de taille (batch_size x data_size)
    - Vecteur labels de taille batch_size
    """
    data_batches = ["data_batch_" + str(i) for i in range(1, 6)] + ['test_batch']

    flag = True

    for db in data_batches:
        data, labels = read_cifar_batch(os.path.join(path, db))
        if flag:
                DATA = data
                LABELS = labels
                flag = False
        else:
            DATA = np.concatenate((DATA, data), axis=0, dtype=np.float32)
            LABELS = np.concatenate((LABELS, labels), axis=-1, dtype=np.int64)
    return DATA, LABELS

def split_dataset(data, labels, split=0.6):
    """
    Cette fonction divise l'ensemble de notre data en training data set et testing data set.

    """
    n = data.shape[0]
    indices = np.random.permutation(n)
    train_idx, test_idx = indices[:int(split*n)], indices[int(split*n):]

    data_train, data_test = data[train_idx,:].astype(np.float32), data[test_idx,:].astype(np.float32)
    labels_train, labels_test = labels[train_idx].astype(np.int64), labels[test_idx].astype(np.int64)

    return data_train, labels_train, data_test, labels_test 